@@ -2,9 +2,11 @@
 
 package Data::ObjectDriver::Driver::Cache::RAM;
 use strict;
+use warnings;
+
 use base qw( Data::ObjectDriver::Driver::BaseCache );
 
-our %Cache;
+my %Cache;
 
 sub init {
     my $driver = shift;
@@ -17,5 +19,6 @@ sub get_from_cache    { $Cache{$_[1]}         }
 sub add_to_cache      { $Cache{$_[1]} = $_[2] }
 sub update_cache      { $Cache{$_[1]} = $_[2] }
 sub remove_from_cache { delete $Cache{$_[1]}  }
+sub clear_cache       { %Cache = ()           }
 
 1;
