@@ -1,7 +1,9 @@
-# $Id: Pg.pm 1082 2006-01-10 22:22:10Z mpaschal $
+# $Id: Pg.pm 169 2006-05-04 00:15:55Z sky $
 
 package Data::ObjectDriver::Driver::DBD::Pg;
 use strict;
+use warnings;
+
 use base qw( Data::ObjectDriver::Driver::DBD );
 
 sub init_dbh {
@@ -16,7 +18,7 @@ sub bind_param_attributes {
     if ($data_type && $data_type eq 'blob') {
         return { pg_type => DBD::Pg::PG_BYTEA() };
     }
-    return undef;
+    return;
 }
 
 sub sequence_name {
